# What is ScaleBot?
ScaleBot is a framework for running scheduled jobs against a cloud. It is built upon the Jenkins CI and Juju modeling tools.

# QuickStart (using Amazon Web Services)
Clone a local copy of the sample ScaleBot configuration repository:
```
$ git clone git://git.launchpad.net/~scalebot-team/+git/scalebot-config
```
Make a copy of the aws credentials template, and add your account keys:
```
$ cd scalebot-config
$ cp labs/aws/credentials.yaml.template labs/aws/credentials.yaml
$ vi labs/aws/credentials.yaml
```
Deploy the ScaleBot bundle. In this example, we'll deploy to local LXD containers:
```
$ juju bootstrap localhost
Creating Juju controller "localhost-localhost" on localhost/localhost
Looking for packaged Juju agent version 2.2.4 for amd64
To configure your system to better support LXD containers, please see: https://github.com/lxc/lxd/blob/master/doc/production-setup.md
Launching controller instance(s) on localhost/localhost...
 - juju-689661-0 (arch=amd64)
Fetching Juju GUI 2.9.2
Waiting for address
Attempting to connect to 10.98.200.35:22
Bootstrap agent now started
Contacting Juju controller at 10.98.200.35 to verify accessibility...
Bootstrap complete, "localhost-localhost" controller now available.
Controller machines are in the "controller" model.
Initial model "default" added.
$ juju deploy bundle.yaml
Deploying charm "cs:xenial/jenkins-4"
Deploying charm "cs:~ce-hyperscale/scalebot-jenkins-3"
Related "jenkins:extension" and "scalebot:extension"
Deploy of bundle completed.
```
Once deployment has completed, use ```juju status``` to find the IP of your scalebot jenkins server. You can connect to it at http://ip-of-server:8080. The sample configuration includes an example "Hello World" job. Building this job will deploy a node in the Lab cloud and run ```echo "Hello World"``` on it.

# Architecture
ScaleBot is deployed from a Juju charm that builds a running instance of Jenkins CI, initialized with a git repository that provides job definitions and a target cloud to run those jobs against. We refer to this target cloud as a "lab". ScaleBot will use the target cloud ("lab") configuration to automatically setup an *internal* Juju controller which Jenkins jobs can use to deploy systems and workloads as necessary within the lab.

<!-- To edit this image, modify the following image and select
     "Publish to the web..."
     https://docs.google.com/drawings/d/1BPKUyUMAy8PfAf81397eAKD8shFQT3D9uuqDW4yfRCg/edit -->

![alt text](https://docs.google.com/drawings/d/e/2PACX-1vS0RwIg_CyMTKzJlnRJTf9wrH4e4v3W8wbMsI4RwEcB1FuvclU8Q7X4NPVOLSzYWtGBcXERMVI-PY7p/pub?w=935&h=527 "ScaleBot Diagram A")

Of course, since scalebot-jenkins is a charm itself, it also is deployed in a cloud. This can just be a LXD cloud though, as the controller and Jenkins server can easily run in containers. The important thing is that the jenkins server in the ScaleBot cloud is on a network that can access the Lab cloud.

<!-- To edit this image, modify the following image and select
     "Publish to the web..."
     https://docs.google.com/drawings/d/1mb8oea6fmZtTmnC-gpMZgmsGnsaMPaO4XELZv0jNRFo/edit -->

![alt text](https://docs.google.com/drawings/d/e/2PACX-1vRNzqlTKIU57N-cdSA1L8M-20zxUbwMxteOAQEs321NrER-EkSEhbb1P-qZc8V_p-dUq1lTmsTl5IHq/pub?w=1016&h=508 "ScaleBot Diagram B")

# Configuration Repository
The scalebot-jenkins charm accepts configuration that tells ScaleBot where to fetch Jenkins job and Lab Cloud definitions. This data must be in a git repository with the following layout:

```
jobs/<jobfoo>.yaml
jobs/<jobbar>.yaml
...
jobs/<jobbaz>.yaml

labs/<labname>/<labname>.yaml
labs/<labname>/clouds.yaml
labs/<labname>/credentials.yaml
labs/<labname>/model-defaults.yaml
scalebot.d/init
scalebot.d/refresh
```

Jobs must be in the YAML format used by [Jenkins Job Builder](https://docs.openstack.org/infra/jenkins-job-builder), as that is the tool used to load them into Jenkins. The clouds.yaml, credentials.yaml, and model-defaults.yaml files are passed directly to Juju when bootsrapping the Lab controller, so see [Juju documentation](http://jujucharms.com/docs) for details on their contents.

You may also want to store code in this repository and call it from your jobs.


# Job Environment
ScaleBot will automatically create an empty model prior to building a jenkins job. This model will have a unique name, based on the $BUILD_ID. Jobs can assume that this model exists and is in-focus prior to executing, so your build code can just begin deploying charms immediately on startup.

Since it is expected that builds will deploy systems using Juju, the python [bindings for juju](https://github.com/juju/python-libjuju) are pre-installed on the system.

## Environment Variables
In addition to the environment variables set by Jenkins, the following variable is also available:

**$SCALEBOT_REPO** will be set to the path of the locally cloned copy of the Scalebot configuration repository.

## scalebot.d/init hook
If it exists, ```$SCALEBOT_REPO/scalebot.d/init``` will be executed each time the configuration repository is refreshed. You can use this to install any additional dependencies needed for your jobs.

## scalebot.d/refresh hook
If it exists, ```$SCALEBOT_REPO/scalebot.d/refresh``` will be executed after the repository is updated, and before loading jobs. This is useful in case something job yaml files need to be dynamically generated.
