#!/usr/bin/env python3
import argparse
import os
import logging
import subprocess
import sys

SCALEBOT_HOME = "/srv/scalebot"
SCALEBOT_REPO = os.path.join(SCALEBOT_HOME, "repo")

def syncjobs(JOBSET):
    jjb = '/usr/local/bin/jenkins-jobs'
    # Load jenkinsjobs config
    config_ini = '/etc/jenkins_jobs/jenkins_jobs.ini'
    #common directory to add jjb Macro
    COMMON = SCALEBOT_REPO + '/jobs/common'
    cmd = [jjb, '--flush-cache', '--conf', config_ini, 'update']
    flag = []
    if os.path.exists(COMMON):
        jobs = '{}/jobs/{}:{}'.format(SCALEBOT_REPO, JOBSET, COMMON)
        flag = ['-r', jobs]
    else:
        jobs = '{}/jobs/{}'.format(SCALEBOT_REPO, JOBSET)
        flag = [jobs]
    subprocess.run(cmd + flag, check=True)

def main():

    parser = argparse.ArgumentParser()

    parser.add_argument('-s', '--syncjobs',
                        help=("This will import jenkins jobs via yaml from \
                                 $SCALEBOT_REPO/jobs/"),
                        required=False,
                        action='store_true')
    parser.add_argument('-d', '--deletejobs',
                        help=("This will Delete all jobs from Hudson, useful \
                              or refreshing job list after branch change."),
                        required=False,
                        action='store_true')
    parser.add_argument('-j', '--jobset',
                        help=("Name of the desired jobset from which to import \
                                 $SCALEBOT_REPO/jobs/$JOBSET"),
                        required=True, type=str)

    args = parser.parse_args()

    # If no args specified upon calling tool, dump help and exit
    if len(sys.argv[1:]) == 0:
        parser.print_help()
        parser.exit()

    try:
        if args.deletejobs:
            # Delete the jobs completely from Jenkins
            logger.info('Removing Jenkins Jobs from Hudson')
            cmd = subprocess.Popen(["jenkins-jobs", "delete-all"],
                                   stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE)
            cmd.communicate(input="Y".encode())
            logger.info('Hudson Jobs Deleted')

        if args.syncjobs:
            JOBSET = args.jobset
            logger.info('Importing Jenkins Jobs')
            syncjobs(JOBSET)

            logger.info('Import Jobs Complete')

    except Exception as e:
        logger.info("Problem with job import")
        logger.info(e)
        return '-1'


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

if __name__ == "__main__":
    sys.exit(main())
